import argparse

help = "Train the RAG system."

parser = argparse.ArgumentParser(
    description = f"{help} Processes documents, extracts knowledge, generates embeddings, and builds a searchable vector database for efficient semantic retrieval and contextual responses",
    add_help = False,
    formatter_class = argparse.ArgumentDefaultsHelpFormatter,
)

subparsers = parser.add_subparsers(
    title = "subcommands",
    dest = "subcommand",
)

subparsers.add_parser(
    "check",
    help = "Check RAG System. Scan all components of the system and diagnose status",
    description = "Check RAG System. Scan all components of the system and diagnose status",
)



import argparse, os
from dotenv import load_dotenv
from loguru import logger

load_dotenv(dotenv_path=os.environ.get('ENV_FILE', '.env'), override=True)

from app.rag.config import top_parser, common_parser, setup_logger
from app.rag.doctor import help as doctor_help, parser as doctor_parser, route as doctor_route
from app.rag.train import help as train_help, parser as train_parser, route as train_route
from app.rag.ask import help as ask_help, parser as ask_parser, route as ask_route
from app.rag.test import help as test_help, parser as test_parser, route as test_route

# Main parser
parser = argparse.ArgumentParser(
    formatter_class = argparse.ArgumentDefaultsHelpFormatter,
    description = 'Pilot of On-Premise RAG',
    parents = [top_parser],
    add_help = False,
)

# Commands
subparsers = parser.add_subparsers(
    title = 'commands',
    dest = 'command',
)

subparsers.add_parser(
    'doctor',
    help = doctor_help,
    description = doctor_parser.description,
    parents = [top_parser, common_parser, doctor_parser],
    add_help = False,
)

subparsers.add_parser(
    'train',
    help = train_help,
    description = train_parser.description,
    parents = [top_parser, common_parser, train_parser],
    add_help = False,
)

subparsers.add_parser(
    'test',
    help = test_help,
    description = test_parser.description,
    parents = [top_parser, common_parser, test_parser],
    add_help = False,
)

subparsers.add_parser(
    'ask',
    help = ask_help,
    description = ask_parser.description,
    parents = [top_parser, common_parser, ask_parser],
    add_help = False,
)

def main():
    args = parser.parse_args()

    command_message = f"with command: {args.command}" if args.command else ""
    logger.info(f"RAG Started {command_message}")
    
    setup_logger(
        log_level = args.log_level,
        log_dir = args.log_dir,
    )
    logger.debug(f"Parsed arguments: {args}")

    if args.command == 'doctor':
        doctor_route(args)
    elif args.command == 'ask':
        ask_route(args)
    elif args.command == 'train':
        train_route(args)
    elif args.command == 'test':
        test_route(args)
    else:
        parser.print_help()

if __name__ == '__main__':
    main()



import argparse

help = "Ask a question to the RAG system."

parser = argparse.ArgumentParser(
    description = f"{help} Queries the knowledge base with natural language questions, retrieves relevant context, and generates accurate, contextually-aware responses based on the retrieved information",
    add_help = False,
    formatter_class = argparse.ArgumentDefaultsHelpFormatter,
)



import argparse

help = "Test the RAG system."

parser = argparse.ArgumentParser(
    description = f"{help} Evaluates system performance by running predefined test cases, measuring accuracy, relevance, and latency metrics to validate retrieval and generation capabilities",
    add_help = False,
    formatter_class = argparse.ArgumentDefaultsHelpFormatter,
)



from langchain_openai.llms import OpenAI
from langchain_openai.embeddings import OpenAIEmbeddings
from pymilvus import MilvusClient

class Agent():
    def __init__(
        self,
        llm_base_url: str,
        llm_api_key: str,
        llm_model: str,
        embedding_base_url: str,
        embedding_api_key: str,
        embedding_model: str,
        milvus_host: str,
        milvus_database: str,
        milvus_collection: str,
    ):
        self.llm_base_url = llm_base_url
        self.llm_api_key = llm_api_key
        self.llm_model = llm_model
        self.embedding_base_url = embedding_base_url
        self.embedding_api_key = embedding_api_key
        self.embedding_model = embedding_model
        self.milvus_host = milvus_host
        self.milvus_database = milvus_database
        self.milvus_collection = milvus_collection
        
        self._llm_client: OpenAI = None
        self._embedding_client: OpenAIEmbeddings = None
        self._milvus_client: MilvusClient = None

    def retrieve_knowledge_base(self, query: str) -> str:
        llm_client = self._get_llm_client()
        embedding_client = self._get_embedding_client()
        milvus_client = self._get_milvus_client()
    
        
    def _get_llm_client(self) -> OpenAI:
        if self._llm_client is None:
            self._llm_client = OpenAI(
                base_url=self.llm_base_url,
                api_key=self.llm_api_key,
                model=self.llm_model,
            )
        return self._llm_client


    def _get_embedding_client(self) -> OpenAIEmbeddings:
        if self._embedding_client is None:
            self._embedding_client = OpenAIEmbeddings(
                base_url=self.embedding_base_url,
                api_key=self.embedding_api_key,
                model=self.embedding_model,
            )
        return self._embedding_client

    def _get_milvus_client(self) -> MilvusClient:
        if self._milvus_client is None:
            self._milvus_client = MilvusClient(
                host=self.milvus_host,
                database=self.milvus_database,
                collection=self.milvus_collection,
            )
        return self._milvus_client



from argparse import Namespace
from ragas import evaluate
from ragas.metrics import (
    answer_relevancy,
    faithfulness,
    context_precision,
    context_recall,
)

def route(args: Namespace):
    
    result = evaluate(
        
    )
    print(args)



import argparse, os, sys
from pathlib import Path
from loguru import logger

# Logger format constants
LOG_TIME_FORMAT = "{time:YYYY-MM-DD HH:mm:ss.SSS!UTC}Z"
LOG_FILE_FORMAT = f"{LOG_TIME_FORMAT} | {{level: <8}} | {{name}}:{{function}}:{{line}} - {{message}}"
LOG_CONSOLE_FORMAT_FULL = f"<green>{LOG_TIME_FORMAT}</green> | <level>{{level: <8}}</level> | <cyan>{{name}}</cyan>:<cyan>{{function}}</cyan>:<cyan>{{line}}</cyan> - <level>{{message}}</level>\n"
LOG_CONSOLE_FORMAT_SIMPLE = f"<green>{LOG_TIME_FORMAT}</green> | <level>{{level: <8}}</level> | <level>{{message}}</level>\n"

# Initial logger setup before setup_logger()
logger.remove()
logger.add(
    sink = sys.stderr,
    level = "INFO", 
    format = lambda record: LOG_CONSOLE_FORMAT_SIMPLE if record["level"].name == "INFO" else LOG_CONSOLE_FORMAT_FULL,
    colorize = True
)

def setup_logger(log_level: str, log_dir: str):
    """Configure logger with specified level and outputs"""
    
    log_dir = Path(log_dir)
    log_dir.mkdir(exist_ok=True, parents=True)
    
    logger.remove()
    
    # File handler
    logger.add(
        sink = log_dir / "{time:YYYYMMDD-HHmmss!UTC}Z.log",
        level = log_level,
        rotation = "100 MB",
        retention = 0,
        format = LOG_FILE_FORMAT,
        serialize = False,
        enqueue = True,
        backtrace = True,
        diagnose = True,
        catch = True
    )
    
    # Console handler
    logger.add(
        sink = sys.stderr,
        level = log_level,
        format = lambda record: LOG_CONSOLE_FORMAT_SIMPLE if record["level"].name == "INFO" else LOG_CONSOLE_FORMAT_FULL,
        colorize = True
    )


class EnvDefault(argparse.Action):
    """Custom argparse action that uses environment variables as defaults.

    This action extends the standard argparse.Action to support reading default values
    from environment variables. If the specified environment variable exists, its value
    will be used as the default value for the argument.

    For boolean flags (store_true/store_false), the environment variable is interpreted
    as a boolean value where 'true', '1', 'yes', or 'on' (case-insensitive) are
    considered True.

    Args:
        envvar (str): Name of the environment variable to use as default
        required (bool, optional): Whether the argument is required. Defaults to True.
            Note: If a default value is found in environment variables, required is set to False.
        default (Any, optional): Default value if environment variable is not set. Defaults to None.
        **kwargs: Additional arguments passed to argparse.Action

    Example:
        ```python
        parser.add_argument(
            '--log-level',
            envvar='LOG_LEVEL',
            help='Logging level',
            default='INFO',
            action=EnvDefault
        )
        ```

    Note:
        The help text is automatically updated to include the environment variable name.
    """
    def __init__(self, envvar, required=True, default=None, **kwargs):
        if envvar and envvar in os.environ:
            env_value = os.environ[envvar]
            # Convert string environment variable to boolean
            if kwargs.get('nargs') is None and kwargs.get('const') is not None:  # store_true/store_false case
                default = env_value.lower() in ('true', '1', 'yes', 'on')
            else:
                default = env_value
            logger.debug(f"Using {envvar}={default} from environment")
        
        if envvar:
            kwargs["help"] += f" (envvar: {envvar})"
        
        if required and default:
            required = False
            
        super(EnvDefault, self).__init__(default=default, required=required, **kwargs)
        self.envvar = envvar

    def __call__(self, parser, namespace, values, option_string=None):
        setattr(namespace, self.dest, values if values is not None else self.default)


# Top-level parser with common options
top_parser = argparse.ArgumentParser(add_help=False)

top_parser.add_argument(
    '-h', '--help',
    help = 'Show help message and exit',
    default = argparse.SUPPRESS,
    action = 'help',
)

top_parser.add_argument(
    '--env-file',
    envvar = 'ENV_FILE',
    help = 'Path to environment file',
    default = '.env',
    type = str,
    action = EnvDefault,
)

top_parser.add_argument(
    '--log-level',
    envvar = 'LOG_LEVEL',
    help = 'Logging level',
    default = 'INFO',
    type = lambda x: x.upper(),
    choices = ['DEBUG', 'INFO', 'WARNING', 'ERROR', 'CRITICAL'],
    required = False,
    action = EnvDefault,
)

top_parser.add_argument(
    '--log-dir',
    envvar = 'LOG_DIR',
    help = 'Path to log directory',
    default = '.pirag/logs',
    type = str,
    required = False,
    action = EnvDefault,
)

top_parser.add_argument(
    '--log-save',
    envvar = 'LOG_SAVE',
    help = 'Save log to file. If this flag is set, the log will be saved to the file specified in the `--log-path`.',
    default = False,
    const = True,
    nargs = 0,
    type = bool,
    required = False,
    action = EnvDefault,
)

common_parser = argparse.ArgumentParser(
    add_help = False,
)



import argparse

from app.rag.config import top_parser, common_parser

help = "Diagnose the RAG system."

parser = argparse.ArgumentParser(
    description = f"{help} Performs comprehensive health checks on all components, validates configurations, and reports issues to ensure optimal system operation",
    add_help = False,
    formatter_class = argparse.ArgumentDefaultsHelpFormatter,
)

subparsers = parser.add_subparsers(
    title = "subcommands",
    dest = "subcommand",
)

subparsers.add_parser(
    "check",
    help = "Check RAG System. Scan all components of the system and diagnose status",
    description = "Check RAG System. Scan all components of the system and diagnose status",
    parents = [top_parser, common_parser],
    add_help = False,
)



from setuptools import setup, find_packages
import os, sys
import tomli

# Load requirements
with open(os.path.join(os.path.dirname(__file__), 'requirements.txt'), 'r') as f:
    requirements = [line.strip() for line in f if line.strip() and not line.startswith('#')]

# Get version from pyproject.toml
with open(os.path.join(os.path.dirname(__file__), '../pyproject.toml'), 'rb') as f:
    pyproject = tomli.load(f)
    version = pyproject["project"]["version"]

APP_NAME = "pirag"

setup(
    name = APP_NAME,
    version = version,
    packages = find_packages(),
    include_package_data = True,
    install_requires = requirements,
    entry_points = {
        "console_scripts": [
            f"{APP_NAME}=main:main",
        ],
    },
)
